import json
import bpy
import bmesh
from mathutils import Vector
from pyproj import Proj, transform

wgs84 = Proj(init="EPSG:4326")
ch1903 = Proj(init="EPSG:21781")
ch_center = Vector((682936.90, 248521.35, 0))

with open("/Users/treyerl/Documents/iA/Teaching/HS14/public_traffic/data/_buildings.json") as bf:
    b = json.load(bf)["elements"]

    relations = [x for x in b if x["type"] == "relation"]
    ways = dict((int(x["id"]), x) for x in b if x["type"] == "way")
    nodes = dict((int(x["id"]), x) for x in b if x["type"] == "node")

    way_refs = []
    for r in relations:
        if "tags" in r:
            if "type" in r["tags"]:
                if r["tags"]["type"] == "multipolygon":
                    curve = bpy.data.curves.new(str(r["id"]), "CURVE")
                    o = bpy.data.objects.new(str(r["id"]), curve)
                    bpy.context.scene.objects.link(o)
                    for way_ref in r["members"]:
                        if way_ref["type"] == "way":
                            ref = way_ref["ref"]
                            way_refs.append(ref)
                            way = ways[ref]
                            node_refs = way["nodes"]
                            spline = curve.splines.new("POLY")
                            spline.points.add(len(node_refs)-1)
                            for i, p in enumerate(spline.points):
                                node = nodes[node_refs[i]]
                                co = Vector(transform(wgs84, ch1903, node["lon"], node["lat"], 0)) - ch_center
                                co.z = 0
                                p.co = co.to_4d()
    bm = bmesh.new()
    for k, way in ways.items():
        if k not in way_refs:
            if "tags" in way:
                if "building" in way["tags"]:
                    print(k)
                    verts = []
                    for node_ref in way["nodes"]:
                        node = nodes[node_ref]
                        co = Vector(transform(wgs84, ch1903, node["lon"], node["lat"], 0)) - ch_center
                        co.z = 0
                        verts.append(bm.verts.new(co))
                    bm.faces.new(verts)

    m = bpy.data.meshes.new("buildings")
    o = bpy.data.objects.new("buildings", m)
    bm.to_mesh(m)
    bpy.context.scene.objects.link(o)

